import urllib
import logging

from zope.interface import classProvides, implements
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.utils import defaultMatcher

from base import AbstractRemoteCommand


class RemoteWorkflowUpdaterSection(AbstractRemoteCommand):
    """
    Do remote workflow state transition using Plone HTTP API.
    Trigger the same HTTP GET query as Workflow menu does in the user
    interface.
    """
    classProvides(ISectionBlueprint)
    implements(ISection)

    def readOptions(self, options):
        """ Read options give in pipeline.cfg
        """
        # Call parent
        AbstractRemoteCommand.readOptions(self, options)
        # Remote site / object URL containing HTTP Basic Auth username and
        # password
        self.pathkey = defaultMatcher(options, 'path-key', self.name, 'path')
        self.transitionskey = defaultMatcher(options, 'transitions-key',
            self.name, 'transitions')

    def __iter__(self):
        self.checkOptions()
        for item in self.previous:
            if not self.target:
                yield item
                continue
            keys = item.keys()
            # Apply defaultMatcher() function to extract necessary data
            # 1) which item will be transitioned
            # 2) with which transition
            pathkey = self.pathkey(*keys)[0]
            transitionskey = self.transitionskey(*keys)[0]
            if not (pathkey and transitionskey):  # not enough info
                yield item
                continue
            path, transitions = item[pathkey], item[transitionskey]
            if isinstance(transitions, basestring):
                transitions = (transitions,)
            #remote_url = urllib.basejoin(self.target, path)

            remote_url = self.target + "/" + path

            if not remote_url.endswith("/"):
                remote_url += "/"

            for transition in transitions:
                transition_trigger_url = urllib.basejoin(remote_url,
                    "content_status_modify?workflow_action=" + transition)
                self.logger.info("%s performing transition '%s'" % (path,
                    transition))
                from httplib import HTTPException
                try:
                    urllib.urlopen(transition_trigger_url)
                    # f = urllib.urlopen(transition_trigger_url)
                    # data = f.read()
                    # XXX Keep going!
                    ## Use Plone not found page signature to detect bad URLs
                    #if "Please double check the web address" in data:
                    #    import pdb ; pdb.set_trace()
                    #    raise RuntimeError("Bad remote URL:" +
                    #transition_trigger_url)

                except HTTPException:
                    # Other than HTTP 200 OK should end up here,
                    # unless URL is broken in which case Plone shows
                    # "Your content was not found page"
                    self.logger.error("fail")
                    msg = "Remote workflow transition failed %s->%s" % (path,
                        transition)
                    self.logger.log(logging.ERROR, msg, exc_info=True)
            yield item
